class JobResultList extends RessourceList {
  constructor(listElement, options = {}) {
    super(listElement, {...JobResultList.options, ...options});
    this.jobId = listElement.dataset.jobId;
    this.user.eventListeners.job.addEventListener((eventType, payload) => this.eventHandler(eventType, payload), this.jobId);
  }

  init() {
    super.init(this.user.data.jobs[this.jobId].results);
  }

  onclick(event) {
    let ressourceElement = event.target.closest('tr');
    if (ressourceElement === null) {return;}
    let jobResultId = ressourceElement.dataset.id;
    let actionButtonElement = event.target.closest('.action-button');
    if (actionButtonElement === null) {return;}
    let action = actionButtonElement.dataset.action;
    switch (action) {
      case 'download':
        window.location.href = this.user.data.jobs[this.jobId].results[jobResultId].download_url;
        break;
      default:
        console.error(`Unknown action: "${action}"`);
        break;
    }
  }

  patch(patch) {
    let re;
    for (let operation of patch) {
      switch(operation.op) {
        case 'add':
          // Matches the only paths that should be handled here: /jobs/{this.user.data.jobs[this.jobId].id}/results/{jobResultId}
          re = new RegExp('^/jobs/' + this.user.data.jobs[this.jobId].id + '/results/(\\d+)$');
          if (re.test(operation.path)) {this.add(operation.value);}
          break;
        default:
          break;
      }
    }
  }

  preprocessRessource(jobResult) {
    let description;
    if (jobResult.filename.endsWith('.pdf.zip')) {
      description = 'PDF files with text layer';
    } else if (jobResult.filename.endsWith('.txt.zip')) {
      description = 'Raw text files';
    } else if (jobResult.filename.endsWith('.vrt.zip')) {
      description = 'VRT compliant files including the NLP data';
    } else if (jobResult.filename.endsWith('.xml.zip')) {
      description = 'TEI compliant files';
    } else if (jobResult.filename.endsWith('.poco.zip')) {
      description = 'HOCR and image files for post correction (PoCo)';
    } else {
      description = 'All result files created during this job';
    }
    return {id: jobResult.id, description: description, filename: jobResult.filename};
  }
}
JobResultList.options = {
  item: `<tr>
           <td><span class="description"></span></td>
           <td><span class="filename"></span></td>
           <td class="right-align">
             <a class="action-button btn-floating tooltipped waves-effect waves-light" data-action="download" data-position="top" data-tooltip="View"><i class="material-icons">file_download</i></a>
           </td>
         </tr>`,
  valueNames: [{data: ['id']}, 'description', 'filename']
};
